==> checkout_server_derive-0.0.1.crate: checkout_server_derive-0.0.1/src/lib.rs <==
extern crate proc_macro;

use proc_macro::TokenStream;
use quote::quote;
use syn;

#[proc_macro_derive(Server)]
pub fn checkout_server_derive(input: TokenStream) -> TokenStream {
    let ast = syn::parse(input).unwrap();

    // Build the trait implementation
    impl_checkout_server(&ast)
}

fn impl_checkout_server(ast: &syn::DeriveInput) -> TokenStream {
    let name = &ast.ident;
    let gen = quote! {
        mod checkout_server {
            pub use checkout_core::{money::Currency, Checkout, Context, ContextProvider, Server, Error};
            pub use checkout_core::dropshot::endpoint;
            pub use checkout_core::dropshot::ApiDescription;
            pub use checkout_core::dropshot::ConfigDropshot;
            pub use checkout_core::dropshot::ConfigLogging;
            pub use checkout_core::dropshot::ConfigLoggingLevel;
            pub use checkout_core::dropshot::HttpError;
            pub use checkout_core::dropshot::HttpResponseOk;
            pub use checkout_core::dropshot::HttpServerStarter;
            pub use checkout_core::dropshot::RequestContext;
            pub use checkout_core::dropshot::TypedBody;
            pub use checkout_core::http::StatusCode;
            pub use checkout_core::serde::{Deserialize, Serialize};
            pub use std::sync::Arc;

            pub struct ServerContext {
                context_provider: #name,
            }

            #[derive(Serialize, Deserialize, JsonSchema)]
            pub struct CreateRequest {
                currency: Currency,
            }

            #[endpoint {
                method = POST,
                path = "/create",
            }]
            pub async fn create_handler(
                req_ctx: Arc<RequestContext<ServerContext>>,
                body_params: TypedBody<CreateRequest>,
            ) -> Result<HttpResponseOk<Checkout>, HttpError> {
                let params = body_params.into_inner();
                let mut ctx = req_ctx.context().context_provider.new_context().await.unwrap();

                ctx.start_transaction().await.unwrap();
                match Checkout::create(&mut ctx, params.currency.clone()).await? {
                    Ok(co) => {
                        ctx.commit_transaction().await.unwrap();
                        Ok(HttpResponseOk(co))
                    }
                    Err(err) => {
                        ctx.abort_transaction().await.unwrap();
                        Err(HttpError {
                            status_code: StatusCode::BAD_REQUEST,
                            error_code: None,
                            external_message: "unavailable".to_string(),
                            internal_message: "unavailable".to_string(),
                        })
                    }
                }
            }
        }

        impl checkout_server::Server for #name {
            type Context = checkout_server::ServerContext;

            fn api_description() -> ApiDescription<checkout_server::ServerContext> {
                let mut api = checkout_server::ApiDescription<checkout_server::ServerContext>::new();
                api.register(checkout_server::create_handler).unwrap();
                api
            }

            fn new_server(self, bind_address: &str) -> Result<checkout_server::HttpServerStarter<checkout_server::ServerContext>, checkout_server::Error> {
                let log = checkout_server::ConfigLogging::StderrTerminal {
                    level: checkout_server::ConfigLoggingLevel::Info,
                }
                .to_logger("minimal-example")
                .map_err(|_| checkout_server::Error {
                    code: "500".to_string(),
                    message: "failed to bind server".to_string()
                })?;

                let api = Self::api_description();

                checkout_server::HttpServerStarter::new(
                    &checkout_server::ConfigDropshot {
                        bind_address: bind_address.parse().unwrap(),
                        request_body_max_bytes: 2048 * 1024,
                    },
                    api,
                    *checkout_server::Arc::new(()),
                    &log,
                )
                .map_err(|_| checkout_server::Error {
                    code: "500".to_string(),
                    message: "failed to bind server".to_string()
                })
            }
        }
    };
    gen.into()
}

==> checkout_server_derive-0.0.2.crate: checkout_server_derive-0.0.2/src/lib.rs <==
extern crate proc_macro;

use proc_macro::TokenStream;
use quote::quote;
use syn;

#[proc_macro_derive(Server)]
pub fn checkout_server_derive(input: TokenStream) -> TokenStream {
    let ast = syn::parse(input).unwrap();

    // Build the trait implementation
    impl_checkout_server(&ast)
}

fn impl_checkout_server(ast: &syn::DeriveInput) -> TokenStream {
    let name = &ast.ident;
    let gen = quote! {
        mod checkout_server {
            pub use checkout_core::{money::Currency, Checkout, Context, ContextProvider, Server, Error};
            pub use checkout_core::dropshot::endpoint;
            pub use checkout_core::dropshot::ApiDescription;
            pub use checkout_core::dropshot::ConfigDropshot;
            pub use checkout_core::dropshot::ConfigLogging;
            pub use checkout_core::dropshot::ConfigLoggingLevel;
            pub use checkout_core::dropshot::HttpError;
            pub use checkout_core::dropshot::HttpResponseOk;
            pub use checkout_core::dropshot::HttpServerStarter;
            pub use checkout_core::dropshot::RequestContext;
            pub use checkout_core::dropshot::TypedBody;
            pub use checkout_core::http::StatusCode;
            pub use checkout_core::serde::{Deserialize, Serialize};
            pub use std::sync::Arc;

            pub struct ServerContext {
                context_provider: #name,
            }

            #[derive(Serialize, Deserialize, JsonSchema)]
            pub struct CreateRequest {
                currency: Currency,
            }

            #[endpoint {
                method = POST,
                path = "/create",
            }]
            pub async fn create_handler(
                req_ctx: Arc<RequestContext<ServerContext>>,
                body_params: TypedBody<CreateRequest>,
            ) -> Result<HttpResponseOk<Checkout>, HttpError> {
                let params = body_params.into_inner();
                let mut ctx = req_ctx.context().context_provider.new_context().await.unwrap();

                ctx.start_transaction().await.unwrap();
                match Checkout::create(&mut ctx, params.currency.clone()).await? {
                    Ok(co) => {
                        ctx.commit_transaction().await.unwrap();
                        Ok(HttpResponseOk(co))
                    }
                    Err(err) => {
                        ctx.abort_transaction().await.unwrap();
                        Err(HttpError {
                            status_code: StatusCode::BAD_REQUEST,
                            error_code: None,
                            external_message: "unavailable".to_string(),
                            internal_message: "unavailable".to_string(),
                        })
                    }
                }
            }
        }

        impl checkout_server::Server for #name {
            type Context = checkout_server::ServerContext;

            fn api_description() -> ApiDescription<checkout_server::ServerContext> {
                let mut api = checkout_server::ApiDescription<checkout_server::ServerContext>::new();
                api.register(checkout_server::create_handler).unwrap();
                api
            }

            fn new_server(self, bind_address: &str) -> Result<checkout_server::HttpServerStarter<checkout_server::ServerContext>, checkout_server::Error> {
                let log = checkout_server::ConfigLogging::StderrTerminal {
                    level: checkout_server::ConfigLoggingLevel::Info,
                }
                .to_logger("minimal-example")
                .map_err(|_| checkout_server::Error {
                    code: "500".to_string(),
                    message: "failed to bind server".to_string()
                })?;

                let api = Self::api_description();

                checkout_server::HttpServerStarter::new(
                    &checkout_server::ConfigDropshot {
                        bind_address: bind_address.parse().unwrap(),
                        request_body_max_bytes: 2048 * 1024,
                    },
                    api,
                    *checkout_server::Arc::new(ServerContext {
                        context_provider: self
                    }),
                    &log,
                )
                .map_err(|_| checkout_server::Error {
                    code: "500".to_string(),
                    message: "failed to bind server".to_string()
                })
            }
        }
    };
    gen.into()
}
